#include "presstempmodule.h"

PressTempModule::PressTempModule(ManageSerialPort *_serialPort, RefreshTimer *_commonRefreshTimer, Module *_module, QWidget *parent) :
    QWidget(parent)
{
    //Constructor
    serialCom=_serialPort;
    commonRefreshTimer=_commonRefreshTimer;
    module=_module;
    channelNamesSignalMapper=NULL;

    fillWindow();

    // In order to map signals for names changed :
    channelNamesSignalMapper= new QSignalMapper(this);
    connect(channelNamesSignalMapper,SIGNAL(mapped(int)),this,SLOT(sendNewName(int)));

    //get channel names
    channelNamesUpdated=updateChannelNames();

    //init timer
    commonRefreshTimer->connectTimeout(this);
    commonRefreshTimer->startSmallInterval();
}

PressTempModule::~PressTempModule()
{   //Destructor
    //TODO : delete ?
}

void PressTempModule::fillWindow()
{
    //Place GUI elements
    globalLayout= new QVBoxLayout;
    setLayout(globalLayout);
        groupBoxModule = new QGroupBox;

        //Module type
        if(module->typeOfModule==pressureModule) groupBoxModule->setTitle("Pressure Module");
        else if(module->typeOfModule==temperatureModule) groupBoxModule->setTitle("Temperature Module");
        else groupBoxModule->setTitle("ERROR");
        globalLayout->addWidget(groupBoxModule);
            verticalLayout= new QVBoxLayout;
            groupBoxModule->setLayout(verticalLayout);

            // Display module SN
                moduleSNLabel = new QLabel("Module : "+QString::number(module->serialNb,16).rightJustified(8,'0'));
                moduleSNLabel->setAlignment(Qt::AlignCenter);
                moduleSNLabel->setMinimumHeight(40);
                verticalLayout->addWidget(moduleSNLabel);

                // Channel names and values
                channelGridLayout = new QGridLayout;
                    for(int channelIndex=0; channelIndex<6; channelIndex++)
                    {
                        QLineEdit *channelNameToAdd = new QLineEdit("xxx");
                        channelNameToAdd->setMinimumWidth(100);
                        channelNamesVector.push_back(channelNameToAdd);
                        channelGridLayout->addWidget(channelNameToAdd,channelIndex,0);
                        QLabel *channelValueToAdd = new QLabel("xxx");
                        channelValueToAdd->setMinimumWidth(90);
                        channelValuesVector.push_back(channelValueToAdd);
                        channelGridLayout->addWidget(channelValueToAdd,channelIndex,1);
                    }
                channelGridLayout->setVerticalSpacing(25);
                verticalLayout->addLayout(channelGridLayout);

                QSpacerItem *spacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);
                verticalLayout->addItem(spacer);

                //Button back
                buttonLayout = new QHBoxLayout;
                    returnButton = new QPushButton("<= Back");
                    returnButton->setMaximumWidth(80);
                    buttonLayout->addWidget(returnButton);
                    connect(returnButton,SIGNAL(clicked()),this,SLOT(backToManageModule()));
                    //TODO : others buttons ?
                    QSpacerItem *spacerButton = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);
                    buttonLayout->addItem(spacerButton);
                verticalLayout->addLayout(buttonLayout);
}

bool PressTempModule::updateChannelNames()
{
    //Get and display channel names

    //reading names :
    uchar channelNames[96];
    bool ok=true;

    //32 bytes (2 names of 16 bytes)
    ok = ok & serialCom->readBytesAtWithRetry(module->serialNb,0x010000E0,4,3);
    if(ok) for(int i=0; i<32; i++) channelNames[i+64]=serialCom->getReceivedData()[i];

    //64 bytes (4 names of 16 bytes)
    ok=ok & serialCom->readBytesAtWithRetry(module->serialNb,0x010000A0,8,3);
    if(ok) for(int i=0; i<64; i++) channelNames[i]=serialCom->getReceivedData()[i];

    if(ok)  //DL success
    {
        for(int channelIndex=0;channelIndex<6;channelIndex++)
        {
            //get name in vector
            channelNamesVector[channelIndex]->disconnect();
            char channelName[17];
            for(int charIndex=0; charIndex<16; charIndex++)
            {
                if((serialCom->getReceivedData()[charIndex]>0x1F)&&(serialCom->getReceivedData()[charIndex]!=0xFF))
                    channelName[charIndex]=channelNames[channelIndex*16+charIndex];
                else {
                    channelName[charIndex]='\0';
                    break;
                }
            }
            //Set name
            module->channelName[channelIndex]=QString::fromStdString(channelName);
            channelNamesVector[channelIndex]->setText(QString::fromStdString(channelName));

            connect(channelNamesVector[channelIndex], SIGNAL(returnPressed()),channelNamesSignalMapper,SLOT(map()));
            channelNamesSignalMapper->setMapping(channelNamesVector[channelIndex],channelIndex);
        }
        return true;
    }
    else return false;
}

//*****************
// Slots

void PressTempModule::backToManageModule()
{
    //Button back clicked, stop refresh and return to module grid
    commonRefreshTimer->disconnectTimeout();
    emit backToMainWindow();    //Signal to restart the main window (refresh Values)
}

void PressTempModule::startTimerAgain()
{
    commonRefreshTimer->connectTimeout(this);
    commonRefreshTimer->startSmallInterval(); // insert a small delay before fetching values
}

void PressTempModule::timerSlot()
{
    //Get new values from module
    commonRefreshTimer->stop();

    //serialCom->deleteErrorCom();    //retry

    if(!channelNamesUpdated) channelNamesUpdated=updateChannelNames();

    //Read current values
    if(serialCom->readBytesAtWithRetry(module->serialNb,0x1E0,2,3))
    {
        switch(module->typeOfModule)
        {
           case pressureModule:
                for(int i=0; i<4; i++)
                {
                    if(i<2) // temperature
                    {
                        channelValuesVector[i]->setText(QString::number(int16(peekUint16FromUcharTabBigEndian(
                                serialCom->getReceivedData(),2*i))/255.0,'f',1)+" C");
                    }
                    else   // pressure
                    {
                        channelValuesVector[i]->setText(QString::number(int16(peekUint16FromUcharTabBigEndian(
                                serialCom->getReceivedData(),2*i))/60.0,'f',1)+" Pa");
                    }
                }
                //Pulses:
                channelValuesVector[4]->setText(QString::number(peekUint16FromUcharTabBigEndian(serialCom->getReceivedData(),8)));
                channelValuesVector[5]->setText(QString::number(peekUint16FromUcharTabBigEndian(serialCom->getReceivedData(),10)));
                break;
            case pulsesModule:
                for(int i=0; i<6; i++)
                {
                    channelValuesVector[i]->setText(QString::number(peekUint16FromUcharTabBigEndian(serialCom->getReceivedData(),2*i)));
                }
            break;


            case temperatureModule:
            default:

                //Temp255 :
                for(int i=0; i<3; i++)
                {
                    channelValuesVector[i]->setText(QString::number(int16(peekUint16FromUcharTabBigEndian(
                                serialCom->getReceivedData(),2*i))/255.0,'f',1)+" C");
                }
                //Temp100
                channelValuesVector[3]->setText(QString::number(int16(peekUint16FromUcharTabBigEndian(
                                serialCom->getReceivedData(),6))/100.0,'f',1)+" C");
                //Hygro
                channelValuesVector[4]->setText(QString::number(int16(peekUint16FromUcharTabBigEndian(
                                serialCom->getReceivedData(),8))/100.0,'f',1)+" %RH");
                //Pulses
                channelValuesVector[5]->setText(QString::number(peekUint16FromUcharTabBigEndian(serialCom->getReceivedData(),10)));

        }
    }
    commonRefreshTimer->startInterval();    //refresh
}

void PressTempModule::sendNewName(int _channel)
{
    //When changed, send a new channel name
    if(serialCom->setModuleName(module->serialNb,0x010000A0+16*_channel,channelNamesVector[_channel]->text()))
        module->channelName[_channel]=channelNamesVector[_channel]->text();
    updateChannelNames();
}
